#!/usr/bin/perl
use DBI;
use IO::Socket;
use vars qw(%config @categories @auction_types %form);
use strict;
print "Content-type: text/html\n\n";

#-###########################################################################
# 
# In accordance with the GPL, this copyright notice MUST remain intact:
#
# EveryAuctionDB Release Version 0.6 (9/30/00)
# Copyright (C) 1999,2000 EverySoft
# Registered with the United States Copyright Office, TX5-040-450
# http://www.everysoft.com/
#
#-###########################################################################
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-###########################################################################
#
# Modification Log (please add new entries to bottom):
#
# * 07/2000
#   Matt Hahnfeld (matth@everysoft.com) - Original Concept and Design
#   Version available from http://www.everysoft.com/
#
# * 07/29/2000
#   Dieter Werner (MailMaster@interwer.com) - Many bug fixes and code cleanup
#   Version available from http://www.everysoft.com/
#
# * MM/YYYY
#   Name (email) - Modification
#   Availability
#
#-###########################################################################

local %config;

#################################################
# Basic Configuration (required)                #
#################################################

# 1. You need to assign either a mail program or
# a mail host so confirmation e-mails can
# be sent out.
# Leave one commented and one uncommented.
#
# YOU NEED EITHER A MAIL PROGRAM
# $config{'mailprog'} = '/usr/lib/sendmail -t';
#
# OR YOU NEED A MAIL HOST (SMTP)

$config{'mailhost'} = 'localhost';

# 2. This line should be your e-mail address

$config{'admin_address'} = 'nobody@yourhost.com';

# 3. Categories (separate using commas)
local @categories = (
'Computers:Pentium III and Above',
'Computers:Pentium II',
'Computers:Pentium',
'Computers:486 and Below',
'Drinks:Beer',
'Drinks:Beer:Cans',
'Drinks:Beer:Bottles',
'Drinks:Soda',
'Drinks:Misc.',
'Electronics',
'Other Stuff'
);

# 4. Script URL (used for outbid and closed e-mail)
$config{'script_url'} = 'http://your.host.com/cgi-bin/auction.pl';

#################################################
# Database Configuration (required)             #
#################################################

# Database Authentication Information
$config{'dbusername'} = 'root';
$config{'dbpassword'} = '';

# Database Connection Information
# MySQL example
$config{'connectstring'} = 'DBI:mysql:database=everyauction:host=localhost';
# PostgreSQL example
# $config{'connectstring'} = 'DBI:postgresql:database=everyauction:host=localhost';
# MS Access example
# $config{'connectstring'} = 'DBI:ODBC:driver=Microsoft Access Driver (*.mdb);dbq=d:\www\www99\auktion18\myauction.mdb';
# MS-SQL Server example
# $config{'connectstring'} = 'DBI:ODBC:everyauction';

#################################################
# Optional Configuration (for power users!)     #
#################################################

# The different types of auctions (only two types are currently supported)
@auction_types = ('Traditional', 'Dutch');

# The number of minutes after the last bid to close an auction (sniper protection)
$config{'snipermin'} = 5;

# The color for cells in program-generated lists (category list, item list)
$config{'heading_cell_color'} = '#999999';
$config{'list_cell_color'} = '#CCCCCC';

# HTML Header
# Check the readme file for available variables and their descriptions
$config{'html_header'} = <<'END_OF_HEADER';
<HTML>
<HEAD><TITLE>[Your Title Here]</TITLE></HEAD>
<BODY BGCOLOR=#FFFFFF>
<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=1 WIDTH=100%><TR><TD ALIGN=LEFT VALIGN=TOP>
<FONT FACE=ARIAL,HELVETICA SIZE=+2>[Your Title Here]</FONT>
</TD><TD ALIGN=RIGHT VALIGN=TOP>
<SEARCH>
</TD></TR></TABLE>
<P>
END_OF_HEADER

# HTML Footer
# Check the readme file for available variables and their descriptions
$config{'html_footer'} = <<'END_OF_FOOTER';
<P><CENTER><TOOLBAR></CENTER>
<P><CENTER><COPYRIGHT>
<BR><CONTACT></CENTER>
</BODY></HTML>
END_OF_FOOTER

# HTML List (Item list, Category List, Etc.)
# Check the readme file for available variables and their descriptions
$config{'html_list'} = <<'END_OF_LIST';
<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=1><TR><TD BGCOLOR=#000000>
<TABLE WIDTH=100% BORDER=0 CELLSPACING=1 CELLPADDING=1><TR><TD BGCOLOR=#FFFFFF><FONT SIZE=+1><B><TITLE></B></FONT>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF>
<LIST>
</TD></TR></TABLE>
</TD></TR></TABLE>
END_OF_LIST

# HTML User Form (Registration, Account Modification)
# Check the readme file for available variables and their descriptions
$config{'html_userform'} = <<'END_OF_USERFORM';
<INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<USERNAME>
<BR><PASSWORD>
<BR><EMAIL>
<HR SIZE=1 NOSHADE>
<FULLNAME>
<BR><STREET>
<BR><CITY>
<BR><STATE>
<BR><ZIP>
<BR><PHONE>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_USERFORM

# HTML Item Form (Posting Items)
# Check the readme file for available variables and their descriptions
$config{'html_itemform'} = <<'END_OF_ITEMFORM';
<INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<TITLE>
<HR SIZE=1 NOSHADE>
<CATEGORY>
<BR><QUANTITY>
<BR><TYPE>
<BR><STARTPRICE>
<BR><RESERVE>
<BR><INCREMENT>
<BR><DAYS>
<BR><IMAGE>
<BR><DESCRIPTION>
<BR><EXTRAS>
<P><DISCLAIMER>
<HR SIZE=1 NOSHADE>
<USERNAME>
<BR><PASSWORD>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_ITEMFORM

# HTML Rating Form
# Check the readme file for available variables and their descriptions
$config{'html_rateform'} = <<'END_OF_RATEFORM';
<INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<USERTORATE>
<BR><RATING>
<BR><COMMENT>
<HR SIZE=1 NOSHADE>
<USERNAME>
<BR><PASSWORD>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_RATEFORM

# HTML User Info Form
# Check the readme file for available variables and their descriptions
$config{'html_infoform'} = <<'END_OF_INFOFORM';
<INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<ITEMNUM>
<BR><BUYSELL>
<HR SIZE=1 NOSHADE>
<USERNAME>
<BR><PASSWORD>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_INFOFORM

# HTML Bid Form (Placing Bids)
# Check the readme file for available variables and their descriptions
$config{'html_bidform'} = <<'END_OF_BIDFORM';
<P><FONT SIZE=+1><B><TITLE></B></FONT>
<P><INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<QUANTITY>
<BR><LESS>
<BR><BID>
<HR SIZE=1 NOSHADE>
<DISCLAIMER>
<HR SIZE=1 NOSHADE>
<USERNAME>
<BR><PASSWORD>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_BIDFORM

# HTML Append Form (Adding Information to Description)
# Check the readme file for available variables and their descriptions
$config{'html_appendform'} = <<'END_OF_APPENDFORM';
<P><FONT SIZE=+1><B><TITLE></B></FONT>
<P><INSTRUCTIONS>
<FORMTOP>
<HR SIZE=1 NOSHADE>
<DESCRIPTION>
<HR SIZE=1 NOSHADE>
<PASSWORD>
<HR SIZE=1 NOSHADE>
<FORMBOTTOM>
END_OF_APPENDFORM

# HTML Item (Display Item)
# Check the readme file for available variables and their descriptions
$config{'html_item'} = <<'END_OF_ITEM';
<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=1><TR><TD BGCOLOR=#000000>
<TABLE WIDTH=100% BORDER=0 CELLSPACING=1 CELLPADDING=2><TR><TD BGCOLOR=#FFFFFF COLSPAN=2><FONT SIZE=+1><B><TITLE></B></FONT>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF VALIGN=CENTER ROWSPAN=8 WIDTH=300>
<CENTER><IMAGE></CENTER>
</TD><TD BGCOLOR=#FFFFFF WIDTH=100%><CATEGORY>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><SELLER>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><OPEN>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><CLOSE>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><QUANTITY>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><TYPE>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><START>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><HIGHBID>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF COLSPAN=2><FONT SIZE=+1><B>Description</B></FONT>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF COLSPAN=2><DESCRIPTION>
</TD></TR></TABLE>
</TD></TR></TABLE>
END_OF_ITEM

# HTML Message (Error messages, Status, Prompts, etc.)
# Check the readme file for available variables and their descriptions
$config{'html_message'} = <<'END_OF_MESSAGE';
<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=1><TR><TD BGCOLOR=#000000>
<TABLE WIDTH=100% BORDER=0 CELLSPACING=1 CELLPADDING=2><TR><TD BGCOLOR=#FFFFFF><FONT SIZE=+1><B><TITLE></B></FONT>
</TD></TR>
<TR><TD BGCOLOR=#FFFFFF><MESSAGE>
</TD></TR></TABLE>
</TD></TR></TABLE>
END_OF_MESSAGE

# HTML Toolbar (Can be entered in header or footer)
$config{'html_toolbar'} = <<"END_OF_TOOLBAR";
<A HREF=$ENV{'SCRIPT_NAME'}?>[categories]</A>
<A HREF=$ENV{'SCRIPT_NAME'}?action=newreg>[new registration]</A>
<A HREF=$ENV{'SCRIPT_NAME'}?action=changereg>[change registration]</A>
<A HREF=$ENV{'SCRIPT_NAME'}?action=newitem>[post an item]</A>
<A HREF=$ENV{'SCRIPT_NAME'}?action=newrate>[rate a user]</A>
<A HREF=$ENV{'SCRIPT_NAME'}?action=userinfo>[user info]</A>
END_OF_TOOLBAR

# Various "icons" (you may define these as any HTML tags, including <IMG SRC>)
$config{'hot_item_icon'} = "<FONT COLOR=#990000 SIZE=-1>[HoT]</FONT>";
$config{'hot_item_bids'} = 10;
$config{'pic_item_icon'} = "<FONT COLOR=#009900 SIZE=-1>[PiC]</FONT>";

# E-mail messages
$config{'pass_email_top'} = 'Thank you for registering to use our auction site!';
$config{'close_email_buyer_top'} = 'Congratulations!  You were the high bidder on an auction item at our auction site.';
$config{'close_email_seller_top'} = 'An item you posted at our auction site has closed.';
$config{'outbid_email_top'} = 'You were outbid on an item at our auction site.';
$config{'lost_password_email_top'} = 'This e-mail was sent to you as a result of a lost password request.  If you did not request this reminder you do not need to worry.  You are the only one receiving this e-mail. ';

#################################################
# Main Program (no need to edit anything)       #
#################################################
local %form = &get_form_data;
&print_header unless ($form{'action'} eq 'megalist');

if    ($form{'action'}   eq 'newreg'           ) { &print_reg_form;        }
elsif ($form{'action'}   eq 'procreg'          ) { &proc_reg;              }
elsif ($form{'action'}   eq 'changereg'        ) { &print_change_reg_form; }
elsif ($form{'action'}   eq 'newitem'          ) { &print_item_form;       }
elsif ($form{'action'}   eq 'procitem'         ) { &proc_item;             }
elsif ($form{'action'}   eq 'newrate'          ) { &print_rate_form;       }
elsif ($form{'action'}   eq 'procrate'         ) { &proc_rate;             }
elsif ($form{'action'}   eq 'rateview'         ) { &print_rate;            }
elsif ($form{'action'}   eq 'userinfo'         ) { &print_user_info_form;  }
elsif ($form{'action'}   eq 'procinfo'         ) { &print_user_info;       }
elsif ($form{'action'}   eq 'procbid'          ) { &proc_bid;              }
elsif ($form{'action'}   eq 'procappend'       ) { &proc_append;           }
elsif ($form{'action'}   eq 'search'           ) { &print_item_list;       }
elsif ($form{'action'}   eq 'item'             ) { &print_item;            }
elsif ($form{'action'}   eq 'lostpass'         ) { &lost_password;         }
elsif ($form{'action'}   eq 'sendpass'         ) { &send_pass;             }
elsif ($form{'action'}   eq 'megalist'         ) { &megalist;              }
elsif ($form{'action'}   eq 'setup-msaccess'   ) { &setup_msaccess;        }
elsif ($form{'action'}   eq 'setup-mysql'      ) { &setup_mysql;           }
elsif ($form{'action'}   eq 'setup-postgresql' ) { &setup_postgresql;      }
elsif ($form{'action'}   eq 'setup-mssql'      ) { &setup_mssql;           }
elsif ($form{'category'} ne ''                 ) { &print_auctions;        }
else                                             { &print_category_list;   }

&print_footer unless ($form{'action'} eq 'megalist');

#################################################
# Print the HTML Header                         #
#################################################
sub print_header {
	my $searchform;
	$searchform = "<FORM ACTION=$ENV{'SCRIPT_NAME'} METHOD=GET>\n<INPUT TYPE=HIDDEN NAME=action VALUE=search><INPUT TYPE=HIDDEN NAME=category VALUE=\"$form{'category'}\"><INPUT TYPE=TEXT NAME=query SIZE=25 MAXLENGTH=25> <INPUT TYPE=SUBMIT VALUE=Search><BR><SELECT NAME=querytype>";
	$searchform .= "<OPTION VALUE=1 SELECTED>Keyword Search (this category)</OPTION>" if $form{'category'};
	$searchform .= "<OPTION VALUE=2 SELECTED>Keyword Search (all categories)</OPTION><OPTION VALUE=3>Seller ID Search</OPTION></SELECT>\n</FORM>";
	&printhtml($config{'html_header'},
	'TOOLBAR', $config{'html_toolbar'},
	'SEARCH', $searchform
	);
}

#################################################
# Print the HTML Footer                         #
#################################################
sub print_footer {
	&printhtml($config{'html_footer'},
	'TOOLBAR', $config{'html_toolbar'},
	'COPYRIGHT', "<FONT SIZE=-1><I>Powered by <A HREF=http://www.everysoft.com/>EveryAuctionDB</A></I></FONT>",
	'CONTACT', "<FONT SIZE=-1><A HREF=\"mailto:$config{'admin_address'}\">Contact Us!</A></FONT>"
	);
}

#################################################
# Print the new registration form               #
#################################################
sub print_reg_form {
	&printhtml($config{'html_userform'},
	'INSTRUCTIONS', 'Enter your personal information below to make an account on our system.  You must have an account if you want to buy or sell items.  Your password will be e-mailed to you at the address you specify.',
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'USERNAME', '<B>Select a Username:</B><BR><INPUT TYPE=TEXT NAME="username" SIZE=12 MAXLENGTH=12>',
	'PASSWORD', '',
	'FULLNAME', '<B>Your Full Name:</B><BR><INPUT TYPE=TEXT NAME="fullname" SIZE=30 MAXLENGTH=255>',
	'STREET', '<B>Your Street Address:</B><BR><INPUT TYPE=TEXT NAME="address" SIZE=30 MAXLENGTH=255>',
	'CITY', '<B>City:</B><BR><INPUT TYPE=TEXT NAME="city" SIZE=15 MAXLENGTH=255>',
	'STATE', '<B>State:</B><BR><INPUT TYPE=TEXT NAME="state" SIZE=2 MAXLENGTH=2>',
	'ZIP', '<B>Zip:</B><BR><INPUT TYPE=TEXT NAME="zip" SIZE=10 MAXLENGTH=10>',
	'PHONE', '<B>Phone (incl. area code):</B><BR><INPUT TYPE=TEXT NAME="phone" SIZE=13 MAXLENGTH=13>',
	'EMAIL', '<B>Valid E-Mail Address:</B><BR><INPUT TYPE=TEXT NAME="email" SIZE=30 MAXLENGTH=255>',
	'FORMBOTTOM', '<INPUT TYPE=HIDDEN NAME=action VALUE=procreg><INPUT TYPE=SUBMIT VALUE=Submit> <INPUT TYPE=RESET VALUE=Clear></FORM>'
	);
}

#################################################
# Print the change registration form            #
#################################################
sub print_change_reg_form {
	my $usernum;
	my $fullname;
	my $street;
	my $city;
	my $state;
	my $zip;
	my $phone;
	$usernum = authenticate($form{'username'},$form{'password'});
	if ($usernum eq "") {
		&authentication_form;
	}
	else {
		my $db = &dbconnect;
		my $dbh=$db->prepare("SELECT usFullname,usStreet,usCity,usState,usZip,usPhone FROM tblUsers WHERE usNum=" . int($usernum));
		$dbh->execute();
		($fullname,$street,$city,$state,$zip,$phone) = $dbh->fetchrow_array();
		&dbdisconnect($db);
		&printhtml($config{'html_userform'},
		'INSTRUCTIONS', 'You may modify your personal information here.  If you enter a new e-mail address you will be assigned a new random password, so be sure your new e-mail address is valid.  Press "submit" when you are finished.',
		'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
		'USERNAME', "<B>New Password:</B><BR><INPUT TYPE=PASSWORD NAME=newpassword SIZE=12 MAXLENGTH=12>",
		'PASSWORD', "<B>New Password (Again):</B><BR><INPUT TYPE=PASSWORD NAME=newpassword2 SIZE=12 MAXLENGTH=12>",
		'FULLNAME', "<B>Your Full Name:</B><BR><INPUT TYPE=TEXT NAME=fullname SIZE=30 MAXLENGTH=255 VALUE=\"$fullname\">",
		'STREET', "<B>Your Street Address:</B><BR><INPUT TYPE=TEXT NAME=address SIZE=30 MAXLENGTH=255 VALUE=\"$street\">",
		'CITY', "<B>City:</B><BR><INPUT TYPE=TEXT NAME=city SIZE=15 MAXLENGTH=255 VALUE=\"$city\">",
		'STATE', "<B>State:</B><BR><INPUT TYPE=TEXT NAME=state SIZE=2 MAXLENGTH=2 VALUE=\"$state\">",
		'ZIP', "<B>Zip:</B><BR><INPUT TYPE=TEXT NAME=zip SIZE=10 MAXLENGTH=10 VALUE=\"$zip\">",
		'PHONE', "<B>Phone (incl. area code):</B><BR><INPUT TYPE=TEXT NAME=phone SIZE=13 MAXLENGTH=13 VALUE=\"$phone\">",
		'EMAIL', "<B>New E-Mail Address:</B><BR><INPUT TYPE=TEXT NAME=email SIZE=30 MAXLENGTH=255>",
		'FORMBOTTOM', "<INPUT TYPE=HIDDEN NAME=action VALUE=procreg><INPUT TYPE=HIDDEN NAME=username VALUE=\"$form{'username'}\"><INPUT TYPE=HIDDEN NAME=password VALUE=\"$form{'password'}\"><INPUT TYPE=SUBMIT VALUE=Submit> <INPUT TYPE=RESET VALUE=Clear></FORM>"
		);
	}
}

#################################################
# Print the new item form                       #
#################################################
sub print_item_form {
	my $categorylist;
	my $typelist;
	my $i;
	$categorylist = '<B>Category:</B><BR><SELECT NAME="category">';
	for ($i=0; $i<=$#categories; $i++) {
		$categorylist .= "<OPTION>$categories[$i]</OPTION>";
	}
	$categorylist .= '</SELECT>';
	$typelist = '<B>Auction Type:</B><BR><SELECT NAME="type">';
	for ($i=0; $i<=$#auction_types; $i++) {
		$typelist .= "<OPTION VALUE=$i>$auction_types[$i]</OPTION>";
	}
	$typelist .= '</SELECT>';
	&printhtml($config{'html_itemform'},
	'INSTRUCTIONS', "Enter the information for the item you want to post.  Once you post your item it will be up for bid and you will not be able to remove it.  You must be a <A HREF=$ENV{'SCRIPT_NAME'}?action=newreg>registered user</A> to post an item.",
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'TITLE', '<B>Item Title:</B><BR><INPUT TYPE=TEXT NAME="title" SIZE=50 MAXLENGTH=50>',
	'CATEGORY', $categorylist,
	'QUANTITY', '<B>Quantity:</B><BR><INPUT TYPE=TEXT NAME="quantity" SIZE=11 MAXLENGTH=11>',
	'TYPE', $typelist,
	'STARTPRICE', '<B>Starting Price:</B><BR>$<INPUT TYPE=TEXT NAME="startprice" SIZE=10 MAXLENGTH=10>',
	'RESERVE', '<B>Reserve Price (Optional):</B><BR>$<INPUT TYPE=TEXT NAME="reserve" SIZE=10 MAXLENGTH=10>',
	'INCREMENT', '<B>Bid Increment:</B><BR>$<INPUT TYPE=TEXT NAME="increment" SIZE=10 MAXLENGTH=10 VALUE="1.00">',
	'DAYS', '<B>Number of Days (1-14):</B><BR><INPUT TYPE=TEXT NAME="days" SIZE=2 MAXLENGTH=2>',
	'IMAGE', '<B>Image (include http://):</B><BR><INPUT TYPE=TEXT NAME="image" SIZE=50 MAXLENGTH=255>',
	'DESCRIPTION', '<B>Description (may include HTML, but not JavaScript):</B><BR><TEXTAREA NAME=description ROWS=5 COLS=50></TEXTAREA>',
	'USERNAME', '<B>Username:</B><BR><INPUT TYPE=TEXT NAME="username">',
	'PASSWORD', "<B>Password:</B><BR><INPUT TYPE=PASSWORD NAME=\"password\"> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>",
	'FORMBOTTOM', '<INPUT TYPE=HIDDEN NAME=action VALUE=procitem><INPUT TYPE=SUBMIT VALUE=Submit> <INPUT TYPE=RESET VALUE=Clear></FORM>'
	);
}

#################################################
# Print the new bid form                        #
#################################################
sub print_bid_form {
	my $itemnum = shift;
	my $lowestbid;
	my $increment;
	my $quantity;
	my $db = &dbconnect;
	($lowestbid,$increment,$quantity) = find_lowest_new_bid($itemnum);
	&dbdisconnect($db);
	&printhtml($config{'html_bidform'},
	'TITLE', 'Place A Bid',
	'INSTRUCTIONS', "Enter the information for your bid.  Once you post your bid it will be official and you will not be able to remove it.  You must be a <A HREF=$ENV{'SCRIPT_NAME'}?action=newreg>registered user</A> to bid on an item.  If you are a first time bidder on this item, the lowest possible winning bid you could post is <B>\$$lowestbid</B>.  Remember, you may bid on up to <B>$quantity</B> items and the bid increment set by the user is <B>\$$increment</B>.",
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'QUANTITY', '<B>Quantity:</B><BR><INPUT TYPE=TEXT NAME="quantity" SIZE=11 MAXLENGTH=11 VALUE=1>',
	'LESS', '<B>Accept Fewer Items?</B><BR><INPUT TYPE=RADIO NAME="less" VALUE=1 CHECKED> Yes <INPUT TYPE=RADIO NAME="less" VALUE=0> No',
	'BID', '<B>Bid Price:</B><BR>$<INPUT TYPE=TEXT NAME="bid" SIZE=10 MAXLENGTH=10>',
	'DISCLAIMER', 'Remember, if you win an auction based on this winning bid you are required to follow through with your bid after the auction.',
	'USERNAME', '<B>Username:</B><BR><INPUT TYPE=TEXT NAME="username">',
	'PASSWORD', "<B>Password:</B><BR><INPUT TYPE=PASSWORD NAME=\"password\"> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>",
	'FORMBOTTOM', "<INPUT TYPE=HIDDEN NAME=action VALUE=procbid><INPUT TYPE=HIDDEN NAME=item VALUE=$form{'item'}><INPUT TYPE=SUBMIT VALUE=\"Submit Bid\"> <INPUT TYPE=RESET VALUE=Clear></FORM>"
	);
}

#################################################
# Print add to description form                 #
#################################################
sub print_append_form {
	my $itemnum = shift;
	my $username = shift;
	&printhtml($config{'html_appendform'},
	'TITLE', 'Add To Item Description',
	'INSTRUCTIONS', "If you are the seller, you may add more information to your item description at any time.  Simply enter your password and the content you want to add.",
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'DESCRIPTION', '<B>Description (may include HTML, but not JavaScript):</B><BR><TEXTAREA NAME=description ROWS=5 COLS=50></TEXTAREA>',
	'PASSWORD', "<B>Seller's Password:</B><BR><INPUT TYPE=PASSWORD NAME=\"password\"> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>",
	'FORMBOTTOM', "<INPUT TYPE=HIDDEN NAME=action VALUE=procappend><INPUT TYPE=HIDDEN NAME=item VALUE=\"$itemnum\"><INPUT TYPE=HIDDEN NAME=username VALUE=\"$username\"><INPUT TYPE=SUBMIT VALUE=\"Submit Description\"> <INPUT TYPE=RESET VALUE=Clear></FORM>"
	);
}

#################################################
# Print rating form                             #
#################################################
sub print_rate_form {
	&printhtml($config{'html_rateform'},
	'INSTRUCTIONS', "This form will allow you to submit a rating for a user based on a transaction you have had with the user.  Before giving a user a negative rating, please try to contact that user and resolve any problems you are having.  Once you place a rating on a user you may not remove it.  You must be a <A HREF=$ENV{'SCRIPT_NAME'}?action=newreg>registered user</A> to rate someone.",
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'USERTORATE', '<B>User to Rate:</B><BR><INPUT TYPE=TEXT NAME="usertorate">',
	'RATING', '<B>Rating:</B><BR><SELECT NAME="rating"><OPTION VALUE=1>Positive</OPTION><OPTION VALUE=0>Neutral</OPTION><OPTION VALUE=-1>Negative</OPTION></SELECT>',
	'COMMENT', '<B>Comment:</B><BR><INPUT TYPE=TEXT NAME="comment" SIZE=60 MAXLENGTH=60>',
	'USERNAME', '<B>Your Username:</B><BR><INPUT TYPE=TEXT NAME="username">',
	'PASSWORD', "<B>Password:</B><BR><INPUT TYPE=PASSWORD NAME=\"password\"> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>",
	'FORMBOTTOM', "<INPUT TYPE=HIDDEN NAME=action VALUE=procrate><INPUT TYPE=SUBMIT VALUE=Submit> <INPUT TYPE=RESET VALUE=Clear></FORM>"
	);
}

#################################################
# Print user info form                          #
#################################################
sub print_user_info_form {
	&printhtml($config{'html_infoform'},
	'INSTRUCTIONS', "This form will allow you to retrieve information about a seller or bidders after an auction closes.",
	'FORMTOP', "<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">",
	'ITEMNUM', '<B>Item Number:</B><BR><INPUT TYPE=TEXT NAME="itemnum" SIZE=20>',
	'BUYSELL', '<B>Your Role:</B><BR><INPUT TYPE=RADIO NAME="buysell" VALUE="buy" CHECKED> I was a bidder.<BR><INPUT TYPE=RADIO NAME="buysell" VALUE="sell"> I was the seller',
	'USERNAME', '<B>Username:</B><BR><INPUT TYPE=TEXT NAME="username">',
	'PASSWORD', "<B>Password:</B><BR><INPUT TYPE=PASSWORD NAME=\"password\"> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>",
	'FORMBOTTOM', "<INPUT TYPE=HIDDEN NAME=action VALUE=procinfo><INPUT TYPE=SUBMIT VALUE=Submit> <INPUT TYPE=RESET VALUE=Clear></FORM>"
	);
}

#################################################
# Print ratings for a user                      #
#################################################
sub print_rate {
	my $list;
	my $username;
	my $rating;
	my $rater;
	my $comment;
	my $totalrating = 0;
	$list = '<TABLE BORDER=0 CELLSPACING=2 CELLPADDING=2 WIDTH=100%>';
	$list .= "<TR><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Rating</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=20%><B>User</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=70%><B>Comments</B></TD></TR>";
	my $db = &dbconnect;
	death("No User Number Specified!") if $form{'usernum'} eq "";
	my $dbh=$db->prepare("SELECT usName FROM tblUsers WHERE usNum=" . int($form{'usernum'}));
	$dbh->execute();
	death("User not found!") unless ($username) = $dbh->fetchrow_array();
	$dbh=$db->prepare("SELECT raRating,usName,raComment FROM tblRatings,tblUsers WHERE raUser=" . int($form{'usernum'}) . " AND usNum=raRater ORDER BY raNum DESC");
	$dbh->execute();
	while (($rating,$rater,$comment) = $dbh->fetchrow_array()) {
		$totalrating += $rating;
		if ($rating==1) {
			$rating="<FONT COLOR=#009900>Positive</FONT>";
		}
		elsif ($rating==0) {
			$rating="<FONT COLOR=#000000>Neutral</FONT>";
		}
		elsif ($rating==-1) {
			$rating="<FONT COLOR=#990000>Negative</FONT>";
		}
		$list .= "<TR><TD BGCOLOR=$config{'list_cell_color'}>$rating</TD><TD BGCOLOR=$config{'list_cell_color'}>$rater</TD><TD BGCOLOR=$config{'list_cell_color'}>$comment</TD></TR>";
	}
	$list .= '</TABLE>';
	&dbdisconnect($db);
	&printhtml($config{'html_list'},
	'TITLE', "Ratings for $username (total $totalrating)",
	'LIST', $list
	);
}

#################################################
# Print user info                               #
#################################################
sub print_user_info {
	my $usernum;
	$usernum = authenticate($form{'username'},$form{'password'});
	if ($usernum eq "") {
		&authentication_form;
	}
	&death('You must enter a valid item number') unless int($form{'itemnum'} > 0);
	&death('You must specify whether you were the buyer or seller.') unless $form{'buysell'};
	my $db = &dbconnect;
	if ($form{'buysell'} eq 'sell') {
		&death("You were not the seller of item number $form{'itemnum'}.") unless $db->selectrow_array("SELECT * FROM tblItems WHERE itNum=".int($form{'itemnum'})." AND itSeller=".int($usernum));
		&dbdisconnect($db);
		&print_user_info_seller;
	}
	else {
		&death("You were not a bidder on item number $form{'itemnum'}.") unless $db->selectrow_array("SELECT * FROM tblBids WHERE biItem=".int($form{'itemnum'})." AND biUser=".int($usernum));
		&dbdisconnect($db);
		&print_user_info_bidder;
	}
}

#################################################
# Print user info - seller                      #
#################################################
sub print_user_info_seller {
	my @lessprint = ('exact qty.','or less');
	my $list = qq|<table border="0" cellspacing="2" cellpadding="2" width="100%">|;
	$list .= "<TR><TD BGCOLOR=$config{'heading_cell_color'}><B>User/Bid</B></TD><TD BGCOLOR=$config{'heading_cell_color'}><B>Contact Information</B></TD></TR>";
	my $db = &dbconnect;
	my $dbh = $db->prepare("SELECT biPrice,biQuantity,biLessFlag,biTime,usName,usFullname,usStreet,usCity,usState,usZip,usPhone,usEmail FROM tblBids,tblUsers WHERE biItem=".int($form{'itemnum'})." AND usNum=biUser ORDER BY biPrice DESC,biQuantity DESC,biTime");
	$dbh->execute;
	while (my ($bid,$quantity,$less,$time,$name,$fullname,$street,$city,$state,$zip,$phone,$email) = $dbh->fetchrow_array) {
		$list .= qq|<tr><td bgcolor="$config{'list_cell_color'}" valign="top"><B>$name</B><br>\$$bid<br>$quantity item(s) $lessprint[$less]<br>|.localtime($time).qq|</td><td bgcolor="$config{'list_cell_color'}" valign="top">$fullname<br>$street<br>$city, $state  $zip<br>Ph. $phone<br><a href="mailto:$email">$email</a></td></tr>|;
	}
	$dbh->finish;
	&dbdisconnect($db);
	$list .= qq|</table>|;
	&printhtml($config{'html_list'},
	'TITLE', "Bidder Information (item $form{'itemnum'})",
	'LIST', $list
	);
}

#################################################
# Print user info - bidder                      #
#################################################
sub print_user_info_bidder {
	my $list = qq|<table border="0" cellspacing="2" cellpadding="2" width="100%">|;
	my $db = &dbconnect;
	my ($name,$fullname,$street,$city,$state,$zip,$phone,$email) = $db->selectrow_array("SELECT usName,usFullname,usStreet,usCity,usState,usZip,usPhone,usEmail FROM tblItems,tblUsers WHERE itNum=".int($form{'itemnum'})." AND usNum=itSeller");
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Username</b></td><td bgcolor="$config{'list_cell_color'}">$name</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Real&nbsp;Name</b></td><td bgcolor="$config{'list_cell_color'}">$fullname</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Address</b></td><td bgcolor="$config{'list_cell_color'}">$street</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>City</b></td><td bgcolor="$config{'list_cell_color'}">$city</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>State</b></td><td bgcolor="$config{'list_cell_color'}">$state</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Zip</b></td><td bgcolor="$config{'list_cell_color'}">$zip</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Phone&nbsp;Number</b></td><td bgcolor="$config{'list_cell_color'}">$phone</td></tr>|;
	$list .= qq|<tr><td bgcolor="$config{'heading_cell_color'}"><b>Email</b></td><td bgcolor="$config{'list_cell_color'}"><a href="mailto:$email">$email</a></td></tr>|;
	&dbdisconnect($db);
	$list .= qq|</table>|;
	&printhtml($config{'html_list'},
	'TITLE', "Seller Information (item $form{'itemnum'})",
	'LIST', $list
	);
}

#################################################
# Print subcategories and items (uses below)    #
#################################################
sub print_auctions {
	my $category;
	my $category_list_flag = 0;
	my $item_list_flag = 0;
	foreach $category (@categories) {
		$category_list_flag = 1 if $category =~ /^$form{'category'}\:.+$/;
		$item_list_flag = 1 if $category eq $form{'category'};
	}
	$category_list_flag and &print_category_list;
	$category_list_flag and $item_list_flag and print "<P>\n";
	$item_list_flag and &print_item_list;
}

#################################################
# Print the list of categories                  #
#################################################
sub print_category_list {
	my $list;
	my $i;
	my $j;
	my $count;
	my @categories_to_print;
	my $category;
	my $title;
	if ($form{'category'}) {
		foreach $category (grep { /^$form{'category'}:/; } @categories) {
			if ($category =~ /^($form{'category'}:[^:]+):/) {
				$category = $1;
			}
			push (@categories_to_print, $category) unless  $categories_to_print[$#categories_to_print] eq $category;
		}
		$title = "Categories in $form{'category'}";
	}
	else {
		foreach $category (@categories) {
			if ($category =~ /^([^:]+):/) {
				$category = $1;
			}
			push (@categories_to_print, $category) unless  $categories_to_print[$#categories_to_print] eq $category;
		}
		$title = "Auction Categories";
	}
	$list = '<TABLE BORDER=0 CELLSPACING=2 CELLPADDING=2 WIDTH=100%>';
	$list .= "<TR><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=40%><B>Category</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Items</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=40%><B>Category</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Items</B></TD></TR>";
	my $db = &dbconnect;
	for ($i=0; $i<=(int($#categories_to_print/2)); $i++) {
		$list .= '<TR>';
		for ($j=$i; $j<=$#categories_to_print; $j+=(int($#categories_to_print/2)+1)) {
			my $dbh=$db->prepare("SELECT COUNT(*) FROM tblItems WHERE itCategory LIKE '" . sqlEncode($categories_to_print[$j]) . "%' AND itClosedflag <> 1");
			$dbh->execute();
			($count) = $dbh->fetchrow_array();
			$list .= "<TD BGCOLOR=$config{'list_cell_color'}><A HREF=$ENV{'SCRIPT_NAME'}?action=itemlist&category=" . urlEncode($categories_to_print[$j]) . ">" . &shortcat($categories_to_print[$j]) . "</A></TD><TD BGCOLOR=$config{'list_cell_color'}>$count</TD>";
		}
		$list .= "</TR>\n";
	}
	$list .= '</TABLE>';
	&dbdisconnect($db);
	&printhtml($config{'html_list'},
	'TITLE', $title,
	'LIST', $list
	);
}

#################################################
# Print the list of items                       #
#################################################
sub print_item_list {
	my $list;
	my $highbid;
	my $itemnum;
	my $itemtitle;
	my $itemqty;
	my $itemclose;
	my $image;
	my $numbids;
	my %tempitemhash;
	my @itemorder;
	my $icons;
	my $db_query;
	my $list_title;
	$list = '<TABLE BORDER=0 CELLSPACING=2 CELLPADDING=2 WIDTH=100%>';
	$list .= "<TR><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=60%><B>Item</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Price</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Bids</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=20%><B>Closes</B></TD></TR>";
	$db_query = "SELECT itNum,itTitle,itQuantity,itClose,itImage FROM tblItems";
	if ($form{'action'} eq 'search' and $form{'querytype'} == 1) {
		$db_query .= " WHERE itClosedflag <> 1 AND POSITION(LOWER('" . sqlEncode($form{'query'}) . "') IN LOWER(itTitle)) AND itCategory LIKE '" . sqlEncode($form{'category'}) . "%' ";
		$list_title = "Search Results ($form{'query'})";
	}
	elsif ($form{'action'} eq 'search' and $form{'querytype'} == 2) {
		$db_query .= " WHERE itClosedflag <> 1 AND POSITION(LOWER('" . sqlEncode($form{'query'}) . "') IN LOWER(itTitle)) ";
		$list_title = "Search Results ($form{'query'})";
	}
	elsif ($form{'action'} eq 'search' and $form{'querytype'} == 3) {
		$db_query .= ",tblUsers WHERE LOWER(usName) = LOWER('" . sqlEncode($form{'query'}) . "') AND itSeller=usNum AND itClosedflag <> 1 ";
		$list_title = "Seller Search Results ($form{'query'})";
	}
	else {
		$db_query .= " WHERE itClosedflag <> 1 AND itCategory = '" . sqlEncode($form{'category'}) . "' ";
		$list_title = "Items in $form{'category'}";
	}
	$db_query .= "ORDER BY itClose";
	my $db = &dbconnect;
	my $dbh=$db->prepare($db_query);
	$dbh->execute();
	while (($itemnum,$itemtitle,$itemqty,$itemclose,$image) = $dbh->fetchrow_array()) {
		@{$tempitemhash{$itemnum}} = ($itemtitle,$itemqty,$itemclose,$image);
		push (@itemorder, $itemnum);
	}
	foreach $itemnum (@itemorder) {
		($itemtitle,$itemqty,$itemclose,$image) = @{$tempitemhash{$itemnum}};
		$dbh=$db->prepare("SELECT COUNT(*) FROM tblBids WHERE biItem=" . int($itemnum));
		$dbh->execute();
		($numbids) = $dbh->fetchrow_array();
		$highbid = find_high_bid($itemnum,$itemqty);
		$icons = "";
		$icons .= $config{'pic_item_icon'} if $image;
		$icons .= $config{'hot_item_icon'} if $numbids >= $config{'hot_item_bids'};
		$list .= "<TR><TD BGCOLOR=$config{'list_cell_color'}><A HREF=$ENV{'SCRIPT_NAME'}?action=item&item=$itemnum>$itemtitle</A> $icons</TD><TD BGCOLOR=$config{'list_cell_color'}>\$$highbid</TD><TD BGCOLOR=$config{'list_cell_color'}>$numbids</TD><TD BGCOLOR=$config{'list_cell_color'}>" . nicetime($itemclose) . "</TD></TR>\n";
	}
	$list .= '</TABLE>';
	&dbdisconnect($db);
	&printhtml($config{'html_list'},
	'TITLE', $list_title,
	'LIST', $list
	);
}

#################################################
# Print an item                                 #
#################################################
sub print_item {
	my $closeditem;
	my $title;
	my $itemnum;
	my $image;
	my $category;
	my $sellernum;
	my $seller;
	my $selleremail;
	my $open;
	my $close;
	my $quantity;
	my $type;
	my $start;
	my $reserve;
	my $desc;
	my $highbid;
	my $bidhistory;
	my $closedflag;
	my $sellerrating;
	my $current_time;
	$current_time = time;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT itNum FROM tblItems WHERE itClosedflag <> 1 AND $current_time > itClose");
	$dbh->execute();
	while (($closeditem) = $dbh->fetchrow_array()) {
		$dbh=$db->prepare("SELECT * FROM tblBids WHERE biItem=" . int($closeditem) . " AND (biTime+($config{'snipermin'}*60)) >= $current_time");
		$dbh->execute();
		close_item($closeditem) unless ($dbh->fetchrow_array());
	}
	$dbh=$db->prepare("SELECT itTitle,itNum,itImage,itCategory,usNum,usName,usEmail,itOpen,itClose,itQuantity,itType,itStart,itReserve,itDesc,itClosedflag FROM tblItems,tblUsers WHERE itNum=" . int($form{'item'}) . " AND usNum=itSeller");
	$dbh->execute();
	death("Item $form{'item'} not found...") unless (($title,$itemnum,$image,$category,$sellernum,$seller,$selleremail,$open,$close,$quantity,$type,$start,$reserve,$desc,$closedflag)=$dbh->fetchrow_array());
	$highbid = find_high_bid($itemnum,$quantity);
	$bidhistory = find_bid_history($itemnum,$quantity,$reserve);
	$sellerrating = find_rating($sellernum);
	$type = $auction_types[$type];
	&dbdisconnect($db);
	if ($image) {
		$image="<A HREF=\"$image\"><IMG SRC=\"$image\" WIDTH=300 BORDER=0></A>";
	}
	else {
		$image="<FONT SIZE=+1>No Image</FONT>";
	}
	&printhtml($config{'html_item'},
	'TITLE',"$title (item $itemnum)",
	'IMAGE',$image,
	'CATEGORY',"<B>Category:</B> <A HREF=$ENV{'SCRIPT_NAME'}?action=itemlist&category=" . urlEncode($category) . ">$category</A>",
	'SELLER',"<B>Seller:</B> <A HREF=mailto:$selleremail>$seller</A> (<A HREF=$ENV{'SCRIPT_NAME'}?action=rateview&usernum=$sellernum>$sellerrating</A>)",
	'OPEN',"<B>Start Time:</B> " . nicetime($open),
	'CLOSE',"<B>Close Time:</B> " . nicetime($close) . "<BR><FONT SIZE=-2>Or $config{'snipermin'} minutes after last bid...</FONT>",
	'QUANTITY',"<B>Quantity:</B> $quantity",
	'TYPE',"<B>Auction Type:</B> $type",
	'START',"<B>Starting Price:</B> \$$start",
	'HIGHBID',"<B>Lowest Winning Bid:</B> \$$highbid",
	'DESCRIPTION',$desc
	);
	print "<p>";
	&printhtml($config{'html_list'},
	'TITLE',"Winning Bids",
	'LIST',$bidhistory
	);
	if ($closedflag) {
		print "<P><FONT SIZE=+2 COLOR=#FF0000><B>This item is now closed...</B></FONT>";
	}
	else {
		&print_bid_form($itemnum);
		&print_append_form($itemnum,$seller);
	}
}

#################################################
# Process registration                          #
#################################################
sub proc_reg {
	my $newpassword;
	death("You must enter a username!") unless $form{'username'};
	death("Your username must not contain any special characters!") if $form{'username'} =~ /\W/;
	death("You must enter your full first and last name!") unless $form{'fullname'} =~ /.+ .+/;
	death("You must enter your street address!") unless $form{'address'};
	death("You must enter your city!") unless $form{'city'};
	death("You must enter your state!") unless $form{'state'};
	death("You must enter your zip code!") unless $form{'zip'} =~ /\d\d\d\d\d/;
	death("You must enter a valid phone number!") unless $form{'phone'} =~ /\d\d\d/;
	if ($form{'password'}) {
		my $usernum = authenticate($form{'username'},$form{'password'});
		my $db = &dbconnect;
		if ($form{'newpassword'}) {
			death("Your new passwords do not match!") if $form{'newpassword'} ne $form{'newpassword2'};
			$db->do("UPDATE tblUsers SET usPassword='" . sqlEncode($form{'newpassword'}) . "' WHERE usNum=" . int($usernum));
		}
		$db->do("UPDATE tblUsers SET usFullname='" . sqlEncode($form{'fullname'}) . "',usStreet='" . sqlEncode($form{'address'}) . "',usCity='" . sqlEncode($form{'city'}) . "',usState='" . sqlEncode($form{'state'}) . "',usZip='" . sqlEncode($form{'zip'}) . "',usPhone='" . sqlEncode($form{'phone'}) . "' WHERE usNum=" . int($usernum));
		if ($form{'email'}) {
			death("You must enter a valid new e-mail address!") unless $form{'email'} =~ /\@/;
			$newpassword = &randompass;
			$db->do("UPDATE tblUsers SET usPassword='" . sqlEncode($newpassword) . "' WHERE usNum=" . int($usernum));
			&sendemail($form{'email'},$config{'admin_address'},'Your Auction Password',"$config{'pass_email_top'}\n\nYour Username: $form{'username'}\nYour Generated Password: $newpassword\n\nYou may change your password at any time using our user editor...");
			&printhtml($config{'html_message'},
			'TITLE', 'Thank You!',
			'MESSAGE', 'Your password is re-generated when you change your e-mail address.  You will receive an e-mail containing your new password shortly!'
			);
		}
		else {
			&printhtml($config{'html_message'},
			'TITLE', 'Thank You!',
			'MESSAGE', 'Your account has been updated!'
			);
		}
		&dbdisconnect($db);
	}
	else {
		death("You must enter a valid e-mail address!") unless $form{'email'} =~ /\@/;
		my $db = &dbconnect;
		my $dbh=$db->prepare("SELECT usNum FROM tblUsers WHERE LOWER(usName) = LOWER('" . sqlEncode($form{'username'}) . "') OR LOWER(usEmail) = LOWER('" . sqlEncode($form{'email'}) ."')");
		$dbh->execute();
		death("The username or e-mail address you chose is already in use!") if $dbh->fetchrow_array();
		$newpassword = &randompass;
		$db->do("INSERT INTO tblUsers (usName,usEmail,usPassword,usFullname,usStreet,usCity,usState,usZip,usPhone) VALUES ('" . sqlEncode($form{'username'}) . "','" . sqlEncode($form{'email'}) . "','" . sqlEncode($newpassword) . "','" . sqlEncode($form{'fullname'}) . "','" . sqlEncode($form{'address'}) . "','" . sqlEncode($form{'city'}) . "','" . sqlEncode($form{'state'}) . "','" . sqlEncode($form{'zip'}) . "','" . sqlEncode($form{'phone'}) . "')");
		&dbdisconnect($db);
		&sendemail($form{'email'},$config{'admin_address'},'Your Auction Password',"$config{'pass_email_top'}\n\nYour Username: $form{'username'}\nYour Generated Password: $newpassword\n\nYou may change your password at any time using our user editor...");
		&printhtml($config{'html_message'},
		'TITLE', 'Thank You!',
		'MESSAGE', 'You will receive an e-mail containing your password shortly!'
		);
	}
}

#################################################
# Process a new item post                       #
#################################################
sub proc_item {
	my $usernum;
	my $current_time;
	$current_time = time;
	death("You must enter a username!") unless $form{'username'};
	death("Your username or password is incorrect!") unless $usernum = authenticate($form{'username'},$form{'password'});
	death("You must enter a title!") unless $form{'title'};
	$form{'title'} =~ s/\>/\&gt\;/g;
	$form{'title'} =~ s/\</\&lt\;/g;
	death("You must enter a category!") unless $form{'category'};
	death("You must enter a quantity!") unless $form{'quantity'} > 0;
	death("You must enter a start price!") unless $form{'startprice'} > 0;
	$form{'reserve'} = 0.00 unless $form{'reserve'};
	$form{'increment'} = 1.00 unless $form{'increment'};
	death("Your reserve price must be at least \$.50!") unless $form{'increment'} > .50;
	death("You must the number of days to run your auction!") unless $form{'days'} > 0 and $form{'days'} < 15;
	death("You must enter a correct image URL or leave the field blank for no image!") unless $form{'image'} =~ /^http\:\/\/.+\.(gif|jpg)$/i or $form{'image'} eq "";
	death("You may not use client-side scripts in your description!") if $form{'description'} =~ /\< *SCRIPT/i;
	death("You must enter a description!") unless $form{'description'};
	$form{'description'} =~ s/\n/ /g;
	my $db = &dbconnect;
	$db->do("INSERT INTO tblItems (itCategory,itType,itTitle,itSeller,itStart,itReserve,itIncrement,itQuantity,itOpen,itClose,itDesc,itImage,itClosedflag) VALUES ('" . sqlEncode($form{'category'}) . "'," . int($form{'type'}) . ",'" . sqlEncode($form{'title'}) . "'," . int($usernum) . "," . dolEncode($form{'startprice'}) . "," . dolEncode($form{'reserve'}) . "," . dolEncode($form{'increment'}) . "," . int($form{'quantity'}) . ",$current_time,$current_time+(" . int($form{'days'}) . "*86400),'" . sqlEncode($form{'description'}) . "','" . sqlEncode($form{'image'}) . "',0)");
	&dbdisconnect($db);
	$form{'category'} = urlEncode($form{'category'});
	&printhtml($config{'html_message'},
	'TITLE', 'Thank You!',
	'MESSAGE', "Your item was posted.  You may want to view the <A HREF=$ENV{'SCRIPT_NAME'}?action=itemlist&category=$form{'category'}>list of items</A> to make sure your item was posted!"
	);

}

#################################################
# Process a new bid                             #
#################################################
sub proc_bid {
	my $usernum;
	my $lowestbid;
	my $increment;
	my $quantity;
	my $bidtime;
	my $oldquantity;
	my $checkbidnum;
	my $checkprice;
	my $checkquantity;
	my $lessflag;
	my $counter = 0;
	my $current_time;
	$current_time = time;
	death("You must enter a username!") unless $form{'username'};
	death("Your username or password is incorrect!") unless $usernum = authenticate($form{'username'},$form{'password'});
	($lowestbid,$increment,$quantity) = find_lowest_new_bid($form{'item'});
	death("There are only $quantity items available and you placed a bid for $form{'quantity'} items!") if $form{'quantity'} > $quantity;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT biPrice,biTime,biQuantity FROM tblBids WHERE biItem=" . int($form{'item'}) . " AND biUser=$usernum");
	$dbh->execute();
	if (($lowestbid,$bidtime,$oldquantity) = $dbh->fetchrow_array()) {
		$lowestbid += $increment unless $form{'quantity'} > $oldquantity;
		$lowestbid = dolEncode($lowestbid);
		$bidtime = "'$bidtime'";
	}
	death("Your bid must be at least \$$lowestbid!") if $form{'bid'} < $lowestbid;
	death("Your bid must be in \$$increment increments based on the first bid!") if (($form{'bid'}-$lowestbid) != 0) and (($form{'bid'}-$lowestbid)/$increment) != int(($form{'bid'}-$lowestbid)/$increment);
	$dbh=$db->prepare("SELECT * FROM tblItems WHERE itNum=" . int($form{'item'}) . " AND $current_time > itClose");
	$dbh->execute();
	if ($dbh->fetchrow_array()) {
		$dbh=$db->prepare("SELECT * FROM tblBids WHERE biItem=$form{'item'} AND (biTime+($config{'snipermin'}*60)) >= $current_time");
		$dbh->execute();
		death('The item you are bidding on appears to be closed!') unless $dbh->fetchrow_array();
	}
	$bidtime ||= $current_time;
	$db->do("DELETE FROM tblBids WHERE biItem=" . int($form{'item'}) . " AND biUser=" . int($usernum));
	$db->do("INSERT INTO tblBids (biItem,biUser,biQuantity,biPrice,biTime,biLessFlag,biOutbidflag) VALUES (" . int($form{'item'}) . "," . int($usernum) . "," . int($form{'quantity'}) . "," . dolEncode($form{'bid'}) . ",$bidtime," . int($form{'less'}) . ",0)");
	$dbh=$db->prepare("SELECT biNum,biPrice,biQuantity,biLessFlag FROM tblBids WHERE biItem=" . int($form{'item'}) . " ORDER BY biPrice DESC,biQuantity DESC,biTime");
	$dbh->execute();
	while (($checkbidnum,$checkprice,$checkquantity,$lessflag) = $dbh->fetchrow_array()) {
		$counter += $checkquantity;
		if (($counter > $quantity) and !($lessflag)) {
			outbid($checkbidnum);
			$counter -= $checkquantity;
			next;
		}
		last if $counter >= $quantity;
	}
	while (($checkbidnum,$checkprice,$checkquantity,$lessflag) = $dbh->fetchrow_array()) {
		outbid($checkbidnum);
	}
	&dbdisconnect($db);
	&printhtml($config{'html_message'},
	'TITLE', 'Thank You!',
	'MESSAGE', "Your bid was placed.  You may want to view the <A HREF=$ENV{'SCRIPT_NAME'}?action=item&item=$form{'item'}>item</A> to check the status of your bid!"
	);
}

#################################################
# Process an added description                  #
#################################################
sub proc_append {
	my $usernum;
	my $description;
	my $time;
	death("You must enter a username!") unless $form{'username'};
	death("Your username or password is incorrect!") unless $usernum = authenticate($form{'username'},$form{'password'});
	death("You may not use client-side scripts in your description!") if $form{'description'} =~ /\< *SCRIPT/i;
	$form{'description'} =~ s/\n/ /g;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT itDesc from tblItems where itNum=" . int($form{'item'}));
	$dbh->execute();
	($description) = $dbh->fetchrow_array();
	$time = scalar(localtime(time));
	$description .= "</TD></TR><TR><TD BGCOLOR=#FFFFFF COLSPAN=2><FONT SIZE=+1><B>Added By Seller $time</B></FONT></TD></TR><TR><TD BGCOLOR=#FFFFFF COLSPAN=2>";
	$description .= $form{'description'};
	$dbh=$db->do("UPDATE tblItems SET itDesc='" . sqlEncode($description) . "' WHERE itNum=" . int($form{'item'}));
	&dbdisconnect($db);
	&printhtml($config{'html_message'},
	'TITLE', 'Thank You!',
	'MESSAGE', "Your new description was added.  You may want to look at <A HREF=$ENV{'SCRIPT_NAME'}?action=item&item=$form{'item'}>the item</A> to confirm your added description."
	);
}

#################################################
# Process a new rating                          #
#################################################
sub proc_rate {
	my $usernum;
	my $user_to_rate;
	death("You must enter a username!") unless $form{'username'};
	death("Your username or password is incorrect!") unless $usernum = authenticate($form{'username'},$form{'password'});
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT usNum FROM tblUsers WHERE usName='" . sqlEncode($form{'usertorate'}) . "'");
	$dbh->execute();
	death("The user you want to rate is not in our database!") unless ($user_to_rate) = $dbh->fetchrow_array();
	death("You may not rate yourself!") if $user_to_rate == $usernum;
	$db->do("INSERT INTO tblRatings (raRating,raComment,raUser,raRater) VALUES ($form{'rating'},'" . sqlEncode($form{'comment'}) . "'," . int($user_to_rate) . "," . int($usernum) . ")");
	&dbdisconnect($db);
	&printhtml($config{'html_message'},
	'TITLE', 'Thank You!',
	'MESSAGE', "Your rating was posted.  You may want to look at <A HREF=$ENV{'SCRIPT_NAME'}?action=rateview&usernum=$user_to_rate>$form{'usertorate'}'s ratings</A> to confirm your rating."
	);
}

#################################################
# Authentication Form                           #
#################################################
sub authentication_form {
	my $authentication_form;
	my $key;
	$authentication_form = "
	You must enter a valid username and password to access this feature.
	If you already entered your username and password you may have mistyped something.
	Please enter your username and password below:
	<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">
	<B>Username:</B> <INPUT TYPE=TEXT NAME=username>
	<BR><B>Password:</B> <INPUT TYPE=PASSWORD NAME=password> <FONT SIZE=-1><A HREF=\"$ENV{'SCRIPT_NAME'}?action=lostpass\">Lost Password?</A></FONT>
	";
	foreach $key(keys %form) {
		$authentication_form .= "<INPUT TYPE=HIDDEN NAME=\"$key\" VALUE=\"$form{$key}\">\n" if ($key ne 'username' and $key ne 'password');
	}
	$authentication_form .= "<BR><INPUT TYPE=SUBMIT VALUE=Submit></FORM>";
	&printhtml($config{'html_message'},
	'TITLE', 'Authentication',
	'MESSAGE', $authentication_form
	);
}

#################################################
# Lost Password Form                            #
#################################################
sub lost_password {
	my $lost_password_form;
	$lost_password_form = "
	Lost your password?  No Problem.
	Just enter your username and e-mail address below and we'll send you a reminder!
	<FORM METHOD=POST ACTION=\"$ENV{'SCRIPT_NAME'}\">
	<B>Username:</B> <INPUT TYPE=TEXT NAME=username>
	<BR><B>E-Mail Address:</B> <INPUT TYPE=TEXT NAME=email SIZE=30 MAXLENGTH=255>
	<BR><INPUT TYPE=HIDDEN NAME=action VALUE=sendpass><INPUT TYPE=SUBMIT VALUE=Submit></FORM>";
	&printhtml($config{'html_message'},
	'TITLE', 'Lost Password Form',
	'MESSAGE', $lost_password_form
	);
}

#################################################
# Lost Password Mailer                          #
#################################################
sub send_pass {
	my $password;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT usPassword FROM tblUsers WHERE LOWER(usName) = LOWER('" . sqlEncode($form{'username'}) . "') AND LOWER(usEmail) = LOWER('" . sqlEncode($form{'email'}) ."')");
	$dbh->execute();
	($password) = $dbh->fetchrow_array();
	&dbdisconnect($db);
	death("An account with that username and e-mail address was not found in our database.") unless $password;
	&sendemail($form{'email'},$config{'admin_address'},"Your Auction Password","$config{'lost_password_email_top'}\n\nUsername: $form{'username'}\nPassword: $password");
	&printhtml($config{'html_message'},
	'TITLE', 'Password Sent!',
	'MESSAGE', "Your password has been sent to $form{'email'}.  It should arrive shortly!"
	);
}

#################################################
# Authenticate a user                           #
#################################################
sub authenticate {
	my ($username,$password) = @_;
	my $usernum;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT usNum FROM tblUsers WHERE LOWER(usName) = LOWER('" . sqlEncode($username) . "') AND LOWER(usPassword) = LOWER('" . sqlEncode($password) ."')");
	$dbh->execute();
	($usernum) = $dbh->fetchrow_array();
	&dbdisconnect($db);
	return $usernum;
}

#################################################
# Connect to the database                       #
#################################################
sub dbconnect {
	my $db = DBI->connect($config{'connectstring'}, $config{'dbusername'}, $config{'dbpassword'});
        return $db;
}

#################################################
# Disconnect from the database                  #
#################################################
sub dbdisconnect {
        $_[0]->disconnect();
}

#################################################
# Print HTML pages based on templates           #
#################################################
sub printhtml {
	my $template_to_parse = shift;
	my @variables = @_;
	my %variablehash;
	my $key;

	%variablehash = @variables;
	foreach $key(keys %variablehash) {
		$template_to_parse =~ s/\<$key\>/$variablehash{$key}/gi;
	}
	print $template_to_parse;
}

#################################################
# Parse Form                                    #
#################################################
sub get_form_data {
        my $temp;
        my $buffer;
        my @data;
        read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
        foreach $temp (split(/&|=/,$buffer)) {
                $temp =~ tr/+/ /;
                $temp =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
		$temp =~ s/[\r\n]/ /g;
                push @data, $temp;
        }
        foreach $temp (split(/&|=/,$ENV{'QUERY_STRING'})) {
                $temp =~ tr/+/ /;
                $temp =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
		$temp =~ s/[\r\n]/ /g;
                push @data, $temp;
        }
        return @data;
}

#################################################
# Find the current lowest winning bid           #
#################################################
sub find_high_bid {
	my $itemnum = shift;
	my $sellingqty = shift;
	my $price;
	my $quantity;
	my $lessflag;
	my $lowestwinningbid=0;
	my $counter=0;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT biPrice,biQuantity,biLessFlag FROM tblBids WHERE biItem=" . int($itemnum) . " ORDER BY biPrice DESC,biQuantity DESC,biTime");
	$dbh->execute();
	while (($price,$quantity,$lessflag) = $dbh->fetchrow_array()) {
		$counter += $quantity;
		if (($counter > $sellingqty) and !($lessflag)) {
			$counter -= $quantity;
			next;
		}
		else {
			$lowestwinningbid = $price;
		}
		return $lowestwinningbid if $counter >= $sellingqty;
	}
	if ($lowestwinningbid) {
		return $lowestwinningbid;
	}
	else {
		$dbh=$db->prepare("SELECT itStart FROM tblItems WHERE itNum=" . int($itemnum));
		$dbh->execute();
		($price) = $dbh->fetchrow_array();
		return $price;
	}
	&dbdisconnect($db);
}

#################################################
# Find the lowest new bid, inc, and qnty        #
#################################################
sub find_lowest_new_bid {
	my $itemnum = shift;
	my $price;
	my $quantity;
	my $increment;
	my $sellingqty;
	my $lessflag;
	my $counter=0;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT itQuantity,itIncrement FROM tblItems WHERE itNum=" . int($itemnum));
	$dbh->execute();
	($sellingqty,$increment) = $dbh->fetchrow_array();
	$dbh=$db->prepare("SELECT biPrice,biQuantity,biLessFlag FROM tblBids WHERE biItem=" . int($itemnum) . " ORDER BY biPrice DESC,biQuantity DESC,biTime");
	$dbh->execute();
	while (($price,$quantity,$lessflag) = $dbh->fetchrow_array()) {
		$counter += $quantity;
		if (($counter > $sellingqty) and !($lessflag)) {
			$counter -= $quantity;
			next;
		}
		if ($counter >= $sellingqty) {
			$price = dolEncode($price+$increment) if ($quantity == $sellingqty);
			return ($price,$increment,$sellingqty);
		}
	}
	$dbh=$db->prepare("SELECT itStart FROM tblItems WHERE itNum=" . int($itemnum));
	$dbh->execute();
	($price) = $dbh->fetchrow_array();
	&dbdisconnect($db);
	return ($price,$increment,$sellingqty);
}

#################################################
# Find a user's rating                          #
#################################################
sub find_rating {
	my $usernum = shift;
	my $db = &dbconnect;
	my ($rating)=$db->selectrow_array("SELECT SUM(raRating) from tblRatings WHERE raUser=" . int($usernum));
	&dbdisconnect($db);
	if ($rating) {
		return $rating;
	}
	else {
		return 0;
	}
}

#################################################
# Find the bid history                          #
#################################################
sub find_bid_history {
	my $itemnum = shift;
	my $sellingqty = shift;
	my $reserve = shift;
	my $bid_history;
	my $price;
	my $quantity;
	my $bidder;
	my $time;
	my $lessflag;
	my $counter=0;
	$bid_history = '<TABLE BORDER=0 CELLSPACING=2 CELLPADDING=2 WIDTH=100%>';
	$bid_history .= "<TR><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=60%><B>Bidder</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Price</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=10%><B>Quantity</B></TD><TD BGCOLOR=$config{'heading_cell_color'} WIDTH=20%><B>Bid Time</B></TD></TR>";
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT usName,biPrice,biQuantity,biTime,biLessFlag FROM tblUsers,tblBids WHERE biItem=" . int($itemnum) . " AND usNum = biUser ORDER BY biPrice DESC,biQuantity DESC,biTime");
	$dbh->execute();
	while (($bidder,$price,$quantity,$time,$lessflag) = $dbh->fetchrow_array()) {
		$counter += $quantity;
		if ($counter > $sellingqty) {
			if ($lessflag) {
				$quantity = $quantity-($counter-$sellingqty) . " of $quantity";
			}
			else {
				$counter -= $quantity;
				next;
			}
		}
		$bid_history .= "<TR><TD BGCOLOR=$config{'list_cell_color'}>$bidder</TD><TD BGCOLOR=$config{'list_cell_color'}>\$$price";
		$bid_history .= "*" if $price < $reserve;
		$bid_history .= "</TD><TD BGCOLOR=$config{'list_cell_color'}>$quantity</TD><TD BGCOLOR=$config{'list_cell_color'}>".nicetime($time)."</TD></TR>";
		last if $counter >= $sellingqty;
	}
	&dbdisconnect($db);
	$bid_history .= "<TR><TD BGCOLOR=$config{'list_cell_color'} COLSPAN=4><I>No bids yet...</I></TD></TR>" unless $counter;
	$bid_history .= '</TABLE><FONT SIZE=-1>An asterisk (*) next to the bid price indicates reserve price not yet met.</FONT>';
	return $bid_history;
}

#################################################
# Close an Item                                 #
#################################################
sub close_item {
	my $itemnum = shift;
	my $bidder;
	my $bidderemail;
	my $seller;
	my $selleremail;
	my $price;
	my $quantity;
	my $message = '';
	my $counter = 0;
	my $sellingqty;
	my $reserve;
	my $reservemsg;
	my $lessmsg;
	my $typemsg = '';
	my $itemtitle;
	my $lessflag;
	my $type;
	my $lowestwinningbid;
	my $db = &dbconnect;
	if ($db->do("UPDATE tblItems SET itClosedflag=1 WHERE itNum=" . int($itemnum) . " AND itClosedflag=0")) {
		my $dbh=$db->prepare("SELECT usName,usEmail,itReserve,itQuantity,itTitle,itType FROM tblUsers,tblItems WHERE itNum=" . int($itemnum) . " AND usNum=itSeller");
		$dbh->execute();
		($seller,$selleremail,$reserve,$sellingqty,$itemtitle,$type) = $dbh->fetchrow_array();
		if ($type) {
			$lowestwinningbid = find_high_bid($itemnum,$sellingqty);
			$typemsg = "\nThis was a $auction_types[$type] auction.  All bidders the price of the lowest winning bid (\$$lowestwinningbid).\n"
		}
		$dbh=$db->prepare("SELECT usName,usEmail,biPrice,biQuantity,biLessFlag FROM tblUsers,tblBids WHERE biItem=" . int($itemnum) . " AND usNum = biUser ORDER BY biPrice DESC,biQuantity DESC,biTime");
		$dbh->execute();
		while (($bidder,$bidderemail,$price,$quantity,$lessflag) = $dbh->fetchrow_array()) {
			$counter += $quantity;
			if ($counter > $sellingqty) {
				if ($lessflag) {
					$quantity = $quantity-($counter-$sellingqty) . " of $quantity";
				}
				else {
					$counter -= $quantity;
					next;
				}
			}
			$reservemsg = '';
			$reservemsg = "\nUnfortunately, your bid did not meet the seller's reserve price...\n" if $reserve and $price < $reserve;
			$price = $lowestwinningbid if $type;
			&sendemail($bidderemail,$selleremail,"Auction Close: $itemtitle","$config{'close_email_buyer_top'}\n\nItem Number: $itemnum\nYour Bid: \$$price\nQuantity: $quantity items\n$reservemsg$typemsg\nPlease contact the seller to make any necessary arrangements:\nSeller Username: $seller\nSeller E-Mail Address: $selleremail\n\nIf you would like mailing information or a telephone number for the seller please visit our user information page at $config{'script_url'}?action=userinfo .  To see the item, visit $config{'script_url'}?action=item&item=$itemnum .");
			$message .= "$bidder ($bidderemail) Quantity: $quantity Price: \$$price\n";
			last if $counter >= $sellingqty;
		}
		$message .= "\n\nThe following list includes ALL bids (both winning and non-winning):\n";
		$dbh=$db->prepare("SELECT usName,usEmail,biPrice,biQuantity,biLessFlag FROM tblUsers,tblBids WHERE biItem=" . int($itemnum) . " AND usNum = biUser ORDER BY biPrice DESC,biQuantity DESC,biTime");
		$dbh->execute();
		while (($bidder,$bidderemail,$price,$quantity,$lessflag) = $dbh->fetchrow_array()) {
			$lessmsg = '';
			$lessmsg = "or less" if $lessflag;
			$reservemsg = '';
			$reservemsg = "\nYou are not required to sell if a bid is under your reserve of \$$reserve.\n" if $reserve;
			$message .= "$bidder ($bidderemail) Quantity: $quantity $lessmsg Price: \$$price\n";
		}
		&sendemail($selleremail,$config{'admin_address'},"Auction Close: $itemtitle","$config{'close_email_seller_top'}\n\nItem Number: $itemnum\n$reservemsg\n$typemsg\nThe following are top bids for your $sellingqty item(s):\n$message\n\nIf no bids are listed, there were no bids on your item.  You should contact each individual bidder to arrange for payment and shipping.  If you would like mailing information or a telephone number for a bidder please visit our user information page at:\n     $config{'script_url'}?action=userinfo\n To see the item, visit:\n     $config{'script_url'}?action=item&item=$itemnum");
	}
	&dbdisconnect($db);
}

#################################################
# Send outbid e-mail                            #
#################################################
sub outbid {
	my $bidnum = shift;
	my $bidder;
	my $bidderemail;
	my $itemtitle;
	my $itemnum;
	my $db = &dbconnect;
	if ($db->do("UPDATE tblBids SET biOutbidflag=1 WHERE biNum=" . int($bidnum) . " AND biOutbidflag=0")) {
		my $dbh=$db->prepare("SELECT usName,usEmail,itTitle,biItem FROM tblUsers,tblBids,tblItems WHERE biNum=" . int($bidnum) . " AND itNum = biItem AND usNum = biUser");
		if (($bidder,$bidderemail,$itemtitle,$itemnum) = $dbh->fetchrow_array()) {
			&sendemail($bidderemail,$config{'admin_address'},"Outbid Notice: $itemtitle","$config{'outbid_email_top'}\n\nItem Number: $itemnum\n\nIf you would like to place a higher bid on the item, please visit $config{'script_url'}?action=item&item=$itemnum .");
		}
	}
	&dbdisconnect($db);
}

#################################################
# Send E-Mail                                   #
#################################################
sub sendemail {
        my ($to,$from,$subject,$message) = @_;
        my $trash;
        if ($config{'mailhost'}) {
		eval('use IO::Socket; 1;') or &death("IO::Socket could not be loaded by the script.  Please see the script documentation for details.  It looks like this server is using perl version $].  IO::Socket may not be included with versions of perl prior to 5.00404."); # don't cause errors on machines where IO::Socket is not available
                my $remote;
                $remote = IO::Socket::INET->new("$config{'mailhost'}:smtp(25)");
                $remote->autoflush();
                print $remote "HELO\r\n";
                $trash = <$remote>;
                print $remote "MAIL From:<$config{'admin_address'}>\r\n";
                $trash = <$remote>;
                print $remote "RCPT To:<$to>\r\n";
                $trash = <$remote>;
                print $remote "DATA\r\n";
                $trash = <$remote>;
                print $remote "From: <$from>\r\nSubject: $subject\r\n\r\n";
                print $remote $message;
                print $remote "\r\n.\r\n";
                $trash = <$remote>;
                print $remote "QUIT\r\n";
        }
        else {
                open MAIL, "|$config{'mailprog'}";
                print MAIL "To: $to\r\nFrom: $from\r\nSubject: $subject\r\n\r\n$message\r\n\r\n";
                close MAIL;
        }
}

#################################################
# Random password generator                     #
#################################################
sub randompass {
	my @passset;
	my $randum_num;
	my $randpass;
        srand(time ^ $$);
        @passset = ('a'..'k', 'm'..'n', 'p'..'z', '2'..'9');
        $randpass = "";
        for (my $i = 0; $i < 8; $i++) {
                $randum_num = int(rand($#passset + 1));
                $randpass .= $passset[$randum_num];
        }
        return $randpass;
}

#################################################
# Display and error message and exit            #
#################################################
sub death {
	my $error = shift;
	$error .= "<P>Hit the back button on your browser to try again...</P>";
	&printhtml($config{'html_message'},
	'TITLE', 'Error!',
	'MESSAGE', $error
	);
	&print_footer;
	print STDERR "$error\n";
	exit(0);
}

#################################################
# Return a time based on sec since 1970         #
#################################################
sub nicetime {
	my $seconds=shift;
	my $i;
	my @timearray;
	my @nowtimearray;
	my @months;
	@months = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	@timearray = localtime($seconds);
	@nowtimearray = localtime(time);
	for ($i=0;$i<4;$i++) {
		$timearray[$i] = "0$timearray[$i]" if length($timearray[$i]) == 1;
		$nowtimearray[$i] = "0$nowtimearray[$i]" if length($nowtimearray[$i]) == 1;
	}
	if (($timearray[4] eq $nowtimearray[4]) and ($timearray[3] eq $nowtimearray[3])) {
		return "Today " . $timearray[2] . ":" . $timearray[1] . ":" . $timearray[0];
	} else {
		return $months[$timearray[4]] . " " . $timearray[3] . " " . $timearray[2] . ":" . $timearray[1] . ":" . $timearray[0];
	}
}

#################################################
# Return the short category name                #
#################################################
sub shortcat {
	my $longname=shift;
	if ($longname =~ /^.+:(.+)$/) {
		return $1;
	}
	else {
		return $longname;
	}
}

#################################################
# Database Functions                            #
#################################################
sub sqlEncode {
        my $toencode = $_[0];
        $toencode =~ s/\\/\\\\/g;
        $toencode =~ s/\'/\\\'/g;
        return $toencode;
}

sub urlEncode {
	my $toencode = shift;
	$toencode=~s/ /+/g;
	$toencode=~s/([^a-zA-Z0-9_.+-])/uc sprintf("%%%02x",ord($1))/eg;
	return $toencode;
}

sub dolEncode {
	my $toencode = shift;
	my @dolamt;
        $toencode =~ s/\,//g; 
        @dolamt = split(/\./, $toencode);
        $dolamt[0] = "0" if (!($dolamt[0]));
        $dolamt[0] = int($dolamt[0]);
        $dolamt[1] = substr($dolamt[1], 0, 2);
        $dolamt[1] = "00" if (length($dolamt[1]) == 0);
        $dolamt[1] = "$dolamt[1]0" if (length($dolamt[1]) == 1);
        return "$dolamt[0].$dolamt[1]";
}

#################################################
# Megalist                                      #
#                                               #
# This function can be called by using          #
# auction.pl?action=megalist                    #
# It will produce a "megalist" compatible       #
# with auction indexing sites like BidFind.     #
#                                               #
# For more information, see www.bidfind.com     #
#################################################
sub megalist {
	my $itemnum;
	my $itemtitle;
	my $itemclose;
	my $itemqty;
	my $itemcat;
	my $highbid;
	my %tempitemhash;
	my @itemorder;
	my $flag=0;
	my $startnum;
	$form{'page'} = int($form{'page'});
	print "<HTML>\n<HEAD>\n<TITLE>BidFind MegaList Page $form{'page'}</TITLE>\n</HEAD>\n<BODY>";
	$startnum=$form{'page'}*100;
	my $db = &dbconnect;
	my $dbh=$db->prepare("SELECT itNum,itTitle,itClose,itQuantity,itCategory FROM tblItems WHERE itClosedflag <> 1 ORDER BY itClose DESC LIMIT $startnum,100");
	$dbh->execute();
	while (($itemnum,$itemtitle,$itemclose,$itemqty,$itemcat) = $dbh->fetchrow_array()) {
		$flag=1;
		@{$tempitemhash{$itemnum}} = ($itemtitle,$itemclose,$itemqty,$itemcat);
		push (@itemorder, $itemnum);
	}
	foreach $itemnum (@itemorder) {
		($itemtitle,$itemclose,$itemqty,$itemcat) = @{$tempitemhash{$itemnum}};
		$highbid = find_high_bid($itemnum,$itemqty);
		if ($itemclose =~ /\d\d\d\d\-(\d\d)\-(\d\d)/) {
			$itemclose = "$1/$2";
		}
		print "<A HREF=\"$config{'script_url'}?action=item&item=$itemnum\">$itemtitle</A><PRICE>\$$highbid<CAT>$itemcat<ENDS>$itemclose<BR>\n";
	}
	&dbdisconnect($db);
	$form{'page'}++;
	print "<A HREF=\"$config{'script_url'}?action=megalist&page=$form{'page'}\">Next 100 Items</A></BODY>\n</HTML>" if $flag;
}

#################################################
# Setup Databases                               #
#################################################

# Microsoft Access Module (Contributed by Dieter Werner)
sub setup_msaccess {
        my $error = "";
        my $db = &dbconnect;
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblItems (itNum integer PRIMARY KEY UNIQUE NOT NULL, itCategory TEXT, itType SMALLINT, itTitle TEXT, itSeller INTEGER, itStart CURRENCY, itReserve CURRENCY, itIncrement CURRENCY, itQuantity INTEGER, itOpen INTEGER, itClose INTEGER, itDesc MEMO, itImage TEXT, itClosedflag SMALLINT)");
        $error .= "<BR>";
        $db->do("CREATE TABLE tblBids (biNum integer PRIMARY KEY UNIQUE NOT NULL, biItem INTEGER, biUser INTEGER, biQuantity INTEGER, biPrice CURRENCY, biMax CURRENCY, biTime INTEGER, biLessFlag SMALLINT, biOutbidflag SMALLINT)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblRatings (raNum integer PRIMARY KEY UNIQUE NOT NULL, raRating SMALLINT, raComment TEXT, raUser INTEGER, raRater INTEGER)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblUsers (usNum INTEGER PRIMARY KEY NOT NULL, usName TEXT, usEmail TEXT, usPassword TEXT, usFullname TEXT, usStreet TEXT, usCity TEXT, usState TEXT, usZip TEXT, usPhone TEXT)");
        $error .= "<BR>";
        $error .= $db->errstr;
        &dbdisconnect($db);
        &printhtml($config{'html_message'},
	'TITLE', 'Setup Complete',
	'MESSAGE', "The databases have been set up for MS Access (ODBC)...  Any errors returned by DBI are listed here: $error"
	);
}

# MySQL Module (EverySoft)
sub setup_mysql {
        my $error = "";
        my $db = &dbconnect;
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblBids (biNum int(11) NOT NULL AUTO_INCREMENT,biItem int(11),biUser int(11),biQuantity int(11),biPrice decimal(9,2),biMax decimal(9,2),biTime int(11),biLessFlag smallint(1),biOutbidflag smallint(1),PRIMARY KEY (biNum),UNIQUE PRI (biNum))");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblItems (itNum int(11) NOT NULL AUTO_INCREMENT,itCategory varchar(255),itType smallint(1),itTitle varchar(255),itSeller int(11),itStart decimal(9,2),itReserve decimal(9,2),itIncrement decimal(9,2),itQuantity int(11),itOpen int(11),itClose int(11),itDesc text,itImage varchar(255),itClosedflag smallint(1),PRIMARY KEY (itNum),UNIQUE PRI (itNum))");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblRatings (raNum int(11) NOT NULL AUTO_INCREMENT,raRating smallint(1),raComment varchar(255),raUser int(11),raRater int(11),PRIMARY KEY (raNum),UNIQUE PRI (raNum))");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblUsers (usNum int(11) NOT NULL AUTO_INCREMENT,usName varchar(255),usEmail varchar(255),usPassword varchar(255),usFullname varchar(255),usStreet varchar(255),usCity varchar(255),usState char(2),usZip varchar(10),usPhone varchar(13),PRIMARY KEY (usNum),UNIQUE PRI (usNum))");
        $error .= "<BR>";
        $error .= $db->errstr;
        &dbdisconnect($db);
        &printhtml($config{'html_message'},
        'TITLE', 'Setup Complete',
        'MESSAGE', "The databases have been set up for MySQL...  Any errors returned by DBI are listed here: $error"
        );
}

# PostgreSQL Module (EverySoft)
sub setup_postgresql {
        my $error = "";
        my $db = &dbconnect;
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblBids (biNum SERIAL primary key,biItem int4,biUser int4,biQuantity int4,biPrice money,biMax money,biTime int8,biLessFlag bool,biOutbidflag bool)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblItems (itNum SERIAL primary key,itCategory varchar(255),itType bool,itTitle varchar(255),itSeller int4,itStart money,itReserve money,itIncrement money,itQuantity int4,itOpen int8,itClose int8,itDesc text,itImage varchar(255),itClosedflag bool)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblRatings (raNum SERIAL primary key,raRating bool,raComment varchar(255),raUser int4,raRater int4)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblUsers (usNum SERIAL primary key,usName varchar(255),usEmail varchar(255),usPassword varchar(255),usFullname varchar(255),usStreet varchar(255),usCity varchar(255),usState char(2),usZip varchar(10),usPhone varchar(13))");
        $error .= "<BR>";
        $error .= $db->errstr;
        &dbdisconnect($db);
        &printhtml($config{'html_message'},
        'TITLE', 'Setup Complete',
        'MESSAGE', "The databases have been set up for PostgreSQL...  Any errors returned by DBI are listed here: $error"
        );
}

# Microsoft SQL Server (Contributed by Dieter Werner and Craig Kiesau)
sub setup_mssql {
        my $error = "";
        my $db = &dbconnect;
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblBids (biNum int NOT NULL PRIMARY KEY IDENTITY,biItem int,biUser int,biQuantity int,biPrice money,biMax money,biTime int,biLessFlag smallint,biOutbidflag smallint)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblItems (itNum int NOT NULL PRIMARY KEY IDENTITY,itCategory varchar(255),itType smallint,itTitle varchar(255),itSeller int,itStart money,itReserve money,itIncrement money,itQuantity int,itOpen int,itClose int,itDesc text,itImage varchar(255),itClosedflag smallint)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblRatings (raNum int NOT NULL PRIMARY KEY IDENTITY,raRating smallint,raComment varchar(255),raUser int,raRater int)");
        $error .= "<BR>";
        $error .= $db->errstr;
        $db->do("CREATE TABLE tblUsers (usNum int NOT NULL PRIMARY KEY IDENTITY,usName varchar(255),usEmail varchar(255),usPassword varchar(255),usFullname varchar(255),usStreet varchar(255),usCity varchar(255),usState char(2),usZip varchar(255),usPhone varchar(255))");
        $error .= "<BR>";
        $error .= $db->errstr;
        &dbdisconnect($db);
        &printhtml($config{'html_message'},
        'TITLE', 'Setup Complete',
        'MESSAGE', "The databases have been set up for MS SQL Server...  Any errors returned by DBI are listed here: $error"
        );
}

